/*
 * Project: SafeBee GPS Tracker
 * Author: Zak Kemble, contact@zakkemble.net
 * Copyright: (C) 2019 by Zak Kemble
 * License: 
 * Web: 
 */

#ifndef GPS_H_
#define GPS_H_

typedef enum {
	GPS_STATE_OFF = 0,
	GPS_STATE_LOCKING,
	GPS_STATE_LOCKED
} gps_state_t;

typedef struct {
	millis_t lastLock;
	gps_state_t state;
	char time[11]; // Real NEO-6M: "123005.00", fake: "123005.000"
	char date[7]; // "051217"
	char latitude[11];
	char latitudeHemisphere;
	char longitude[12];
	char longitudeMeridian;
	uint8_t deg_lat_isNegZero; // Can't store negative zero in an integer, so use this flag
	int16_t deg_lat; // -180 to +180
	uint32_t deg_lat_frac;
	uint8_t deg_lon_isNegZero; // Can't store negative zero in an integer, so use this flag
	int8_t deg_lon; // -90 to +90
	uint32_t deg_lon_frac;
	uint8_t speed;
} gps_t;

void gps_init(void);
gps_t* gps_get(uint8_t* isNewData);
void gps_on(void);
void gps_off(void);
void gps_update(void);

#endif /* GPS_H_ */
