/*
 * Project: SafeBee GPS Tracker
 * Author: Zak Kemble, contact@zakkemble.net
 * Copyright: (C) 2019 by Zak Kemble
 * License: 
 * Web: 
 */

#ifndef GSM_H_
#define GSM_H_

#define GSM_CMD_MAX_LEN 32 // Max length of a command

typedef enum{
	GSM_CMD_IDLE = 0,
	GSM_CMD_SKIPDELAY,
	GSM_CMD_DELAY, // Delay a new ms before sending so module comes out of power save mode
	GSM_CMD_SENDING, // Sent command, now waiting for the echo
	GSM_CMD_WAITING // Got the echo, now waiting for response
} gsmCmdState_t;

typedef enum{
	GSM_CMDRESULT_OK = 0, // Command completed with OK
	GSM_CMDRESULT_DATA, // Command is sending response
	GSM_CMDRESULT_ERROR, // Command completed with ERROR
	GSM_CMDRESULT_TIMEOUT // Command took too long
} gsmCmdResult_t;

typedef enum {
	GSM_STATE_OFF = 0,
	GSM_STATE_CHANGING,
	GSM_STATE_TURNINGON,
	GSM_STATE_ON,
	GSM_STATE_POWERDOWN,
	GSM_STATE_POWERINGDOWN
} gsm_state_t;

typedef void (*gsmcmd_callback_t)(char* line, gsmCmdResult_t result);

void gsm_init(void);
void gsm_update(void);
char* gsm_getBuffer(void);
void gsm_sendRaw(char* data, uint8_t len);
uint8_t gsm_command(char* strCommand, millis_t timeout, gsmcmd_callback_t callback);
void gsm_commandUpdateTimeout(millis_t timeout);
void gsm_startup(void);
void gsm_on(void);
void gsm_off(void);
gsm_state_t gsm_getState(void);
uint8_t gsm_isConnected(void);
uint8_t gsm_signal(void);

#endif /* GSM_H_ */
