/*
 * Project: SafeBee GPS Tracker
 * Author: Zak Kemble, contact@zakkemble.net
 * Copyright: (C) 2019 by Zak Kemble
 * License: 
 * Web: 
 */

#ifndef SMS_H_
#define SMS_H_

#define SMS_MESSAGE_LEN 161 // SMS max length 160 + 1 for null terminator
#define SMS_NUMBER_LEN 16  // Max number length = 15 + 1 for null terminator (no plus symbol used in PDU) NOTE: This must be an even value

// https://web.archive.org/web/20080113172956/http://www.dreamfabric.com:80/sms/type_of_address.html
typedef enum{
	NUMBERTYPE_INTERNATIONAL = 0x91,
	NUMBERTYPE_UNKNOWN = 0x81 // National
} numberType_t;

typedef enum{
	SMS_SEND_SUCCESS = 0,
	SMS_SEND_FAILED,
	SMS_SEND_TIMEOUT
} smsSendResult_t;

typedef enum {
	SMS_STATE_IDLE = 0,
	SMS_STATE_SENDSETUP, // Send AT+CMGSto begin sending SMS
	SMS_STATE_PROMT, // Waiting for promt ('>')
	SMS_STATE_SENDING, // Got promt, sent SMS data and now SMS is sending
	SMS_STATE_LISTING, // AT+CMGL sent, waiting for response to get SMS ID
	SMS_STATE_READ, // Wait for AT+CMGL to finish then send AT+CMGR to read single SMS
	SMS_STATE_READING, // Waiting for AT+CMGR response
	SMS_STATE_GETPDU, // Next line will have PDU
	SMS_STATE_GOTPDU, // Got PDU, waiting for AT+CMGR completion
	SMS_STATE_DELETE, // Send AT+CMGD to delete all read messages
	SMS_STATE_DELETEING, // AT+CMGD sent, waiting for completion
	SMS_STATE_DOCALLBACK // AT+CMGD finished, now run callback
} smsState_t;

typedef struct {
	numberType_t type;
	char number[SMS_NUMBER_LEN];
} phoneNumber_t;

typedef struct {
	smsState_t state;
	phoneNumber_t phone;
	char message[SMS_MESSAGE_LEN];
	millis_t timer;
} sms_t;

typedef void (*smssent_callback_t)(smsSendResult_t result);

void sms_init(void);
void sms_notification(void);
sms_t* sms_getBuffer(void);
smsState_t sms_state(void);
uint8_t sms_busy(void);
uint8_t sms_send(phoneNumber_t* phone, smssent_callback_t callback);
void sms_update(uint8_t processNewSMS);

#endif /* SMS_H_ */
