/*
 * Project: SafeBee GPS Tracker
 * Author: Zak Kemble, contact@zakkemble.net
 * Copyright: (C) 2019 by Zak Kemble
 * License: 
 * Web: 
 */

#include "inc/common.h"

// WWDG counter decrements every 4.096ms ((1 / 8MHz)  * 4096 * 8)
// System reset at 200ms
// Refresh window 98ms - 200ms
#define WWDG_REFRESH 48
#define WWDG_WINDOW 24
#define WWDG_INTERVAL 100

#if ENABLE_WWDG
static millis_t lastWWDG;
#endif

#if ENABLE_PRINTF_UART
static void stdout_putf(void* unused, char c)
{
	UNUSED(unused);

	uart_printf(1);
	//putchar(c);
	uart_send(&c, 1);
	uart_printf(0);
}
#endif

static void init(void)
{
	SystemCoreClockUpdate();

#if ENABLE_PRINTF_UART
	init_printf(NULL, stdout_putf);
#endif

	tracker_init();

	uint32_t resetReason = RCC->CSR & (RCC_CSR_LPWRRSTF | RCC_CSR_WWDGRSTF | RCC_CSR_IWDGRSTF | RCC_CSR_SFTRSTF | RCC_CSR_PORRSTF | RCC_CSR_PINRSTF | RCC_CSR_OBLRSTF | RCC_CSR_V18PWRRSTF);
	RCC->CSR |= RCC_CSR_RMVF;
	PRINTD("\n\nRST: 0x%08lX\n\n", resetReason);

#if ENABLE_WWDG
	if(resetReason & RCC_CSR_WWDGRSTF)
	{
		PRINTD("\n\n*****\nWARNING\n\nSYSTEM WAS RESET BY WWDG\n*****\n\n");
		//while(1);
	}

	RCC->APB1ENR |= RCC_APB1ENR_WWDGEN;
	RCC->APB1RSTR |= RCC_APB1RSTR_WWDGRST;
	RCC->APB1RSTR &= ~RCC_APB1RSTR_WWDGRST;
	WWDG->CFR = (WWDG_CFR_WDGTB_1 | WWDG_CFR_WDGTB_0) | 0x40 | WWDG_WINDOW;
	WWDG->CR = WWDG_CR_WDGA | 0x40 | WWDG_REFRESH;
	lastWWDG = millis();
#else
	PRINTD("\n\n*****\nWARNING\n\nWWDG DISABLED\n*****\n\n");
#endif
}

static void update(void)
{
	//static uint32_t loopCounter;

	//static millis_t maxRunTime;

	//millis_t start = millis();
	tracker_update();
	//loopCounter++;
	//millis_t end = millis();

	//millis_t runTime = end - start;
	//if(runTime > maxRunTime)
	//	maxRunTime = runTime;
/*
	static millis_t lastSec;
	if(millis() - lastSec >= 1000)
	{
		PRINTD("LOOPS: %lu\n", loopCounter);
		lastSec += 1000;
		loopCounter = 0;
	}
*/
#if ENABLE_WWDG
	//PRINTD("RUN: %lu\n", end - start);
	//PRINTD("WWDG %lu\n", WWDG->CR & 0x3F);

	millis_t now = millis();
	if(now - lastWWDG >= WWDG_INTERVAL)
	{
		//PRINTD("MAXRUN: %lu\n", maxRunTime);
		//PRINTD("WWDG CLEAR (%lums)\n", now - lastWWDG);
		lastWWDG = now;
		WWDG->CR = WWDG_CR_WDGA | 0x40 | WWDG_REFRESH;
	}
#endif
}

int main(void)
{
	init();

	while(1)
		update();
}
