/*
 * Project: SafeBee GPS Tracker
 * Author: Zak Kemble, contact@zakkemble.net
 * Copyright: (C) 2019 by Zak Kemble
 * License: 
 * Web: 
 */

#include "inc/common.h"

#define ADDR 0x98
#define INTERRUPT_PORT	A
#define INTERRUPT_PIN 	1

static volatile uint8_t shake;

void accel_init()
{
	// A1 = Interrupt
	// B6 = SCL
	// B7 = SDA

	GPIO_SET_PUPD(INTERRUPT_PORT, INTERRUPT_PIN, UP);

	uint8_t buff[4];

	// Standby mode
	buff[0] = 0x07;
	buff[1] = 0b00000000;
	i2c_write(ADDR, buff, 2, 1);

	// Configure
	buff[0] = 0x06;
	buff[1] = 0b11100011; // Interrupt on shake X, Y or Z and orientation change
	buff[2] = 0b00000000;
	buff[3] = 0b00011100; // 8 samples per second
	i2c_write(ADDR, buff, 4, 1);

	// Active mode
	buff[0] = 0x07;
	buff[1] = 0b00000001;
	i2c_write(ADDR, buff, 2, 1);

	// Enable interrupt stuff
	NVIC_SetPriority(EXTI0_1_IRQn, 3);
	NVIC_EnableIRQ(EXTI0_1_IRQn);
	EXTI->IMR |= EXTI_IMR_IM1;
	EXTI->FTSR |= EXTI_FTSR_FT1;
}

void accel_update()
{
/*
	static millis_t lastRun;
	if(millis() - lastRun >= 100)
	{
		lastRun = millis();

		int8_t buff[3];
		buff[0] = 0x00;
		buff[1] = 0x00;
		buff[2] = 0x00;
		i2c_write(ADDR, buff, 1, 0);

		uint8_t count = i2c_read(ADDR, buff, 3);
		UNUSED(count);

		// Values are 6-bit signed values, but the upper 2 bits are always 0
		// So if the sign bit is set then set the upper 2 bits to make a signed 8-bit value
		for(uint8_t i=0;i<3;i++)
		{
			if(buff[i] & 0x20)
				buff[i] |= 0xC0;
		}

		PRINTD("ACCEL: %u %d %d %d\n", count, buff[0], buff[1], buff[2]);
	}
*/
}

uint8_t accel_shake(uint8_t clear)
{
	uint8_t tmpTrig = shake;

	if(clear)
		shake = 0;

	if(!(GPIOA->IDR & GPIO_IDR_1))
	{
		// Clear interrupt by doing a single I2C write
		uint8_t tmp = 0;
		i2c_write(ADDR, &tmp, 1, 1);
	}

	return tmpTrig;
}

void EXTI0_1_IRQHandler(void)
{
	if(EXTI->PR & EXTI_PR_PIF1)
	{
		EXTI->PR |= EXTI_PR_PIF1;
		shake = 1;
	}
}
