/*
 * Project: SafeBee GPS Tracker
 * Author: Zak Kemble, contact@zakkemble.net
 * Copyright: (C) 2019 by Zak Kemble
 * License: 
 * Web: 
 */

#include "inc/common.h"

void adc_init()
{
	// Clocks
	RCC->APB2ENR |= RCC_APB2ENR_ADCEN; // Enable clock
	RCC->APB2RSTR |= RCC_APB2RSTR_ADCRST; // Pulse reset
	RCC->APB2RSTR &= ~RCC_APB2RSTR_ADCRST; // Pulse reset

	// Setup
	ADC1->CFGR1 = ADC_CFGR1_RES_10BIT;
	ADC1->CFGR2 = ADC_CFGR2_CKMODE_PCLK2;
	ADC1->SMPR = ADC_SMPR_SMP_239DOT5; // Make sure this is slow enough for internal temperature sensor
	//ADC1->CHSELR = ADC_CHSELR_CHSEL0;
}

void adc_enable()
{
	if(ADC1->CR & ADC_CR_ADEN) // Already enabled
		return;

	// Calibrate
	ADC1->CR |= ADC_CR_ADCAL;
	while(ADC1->CR & ADC_CR_ADCAL);

	// Enable ADC
	ADC1->CR = ADC_CR_ADEN;
	while(!(ADC1->ISR & ADC_ISR_ADRDY));
//	ADC1->ISR |= ADC_ISR_ADRDY;
}

void adc_disable()
{
	// Stop ADC conversion
//	ADC1->CR |= ADC_CR_ADSTP;
//	while(ADC1->CR & ADC_CR_ADSTP);

	// Disable ADC
	ADC1->CR |= ADC_CR_ADDIS;
	while(ADC1->CR & ADC_CR_ADDIS);
}

uint32_t adc_read(uint8_t channel)
{
	ADC1->CHSELR = 1<<channel;

	ADC1->CR |= ADC_CR_ADSTART;
	while(ADC1->CR & ADC_CR_ADSTART);

	uint32_t val = ADC1->DR;

	return val;
}

#define TEMP30_CAL_ADDR ((uint16_t*)((uint32_t)0x1FFFF7B8)) // Location of calibration values (from the datasheet)
#define VDD_CALIB (3300) // Voltage that CAL was performed at (from the datasheet)
#define VDD_APPLI (3000) // Supply voltage of the MCU at the moment
#define AVG_SLOPE (4300) //AVG_SLOPE from datasheet in microvolts

int32_t adc_temperature()
{
	adc_enable();
	ADC->CCR |= ADC_CCR_TSEN; // Enable temperature sensor

	// Wait at least 10us before reading
	// Around 12us @ 8MHz
	for(uint8_t i=0;i<14;i++)
		__asm__ __volatile__ ("nop");

	uint32_t adc = adc_read(16);

	ADC->CCR &= ~ADC_CCR_TSEN; // Disable temperature sensor
	adc_disable();

	// 'adc' is shifted because the ADC is configured for 10-bit, but the calculations want 12-bit

	int32_t temperature;
	temperature = (((uint32_t)*TEMP30_CAL_ADDR) - (adc<<2) * VDD_APPLI / VDD_CALIB) * 1000;
	temperature /= AVG_SLOPE;
	temperature += 25;

	return temperature;
}
