/*
 * Project: SafeBee GPS Tracker
 * Author: Zak Kemble, contact@zakkemble.net
 * Copyright: (C) 2019 by Zak Kemble
 * License: 
 * Web: 
 */

#include "inc/common.h"

#define ADC_CHANNEL			4
#define UPDATE_INTERVAL		30

#define MAX_VOLTAGE	4200 // Max battery voltage

#define R1			10000 // R1 resistance
#define R2			10000 // R2 resistance
#define VREF		3000 // Reference voltage (VCC)

#define MAX_ADCVAL	((uint16_t)((((R2 / (float)(R1 + R2)) * MAX_VOLTAGE) / VREF) * ADC_MAX))

// Resolution = MAX_VOLTAGE / MAX_ADCVAL
// Resolution = 5.87mV

static uint32_t voltage;
static uint8_t lastSecs;
static uint8_t changeCount;

void battery_init()
{
	// A4 = ADC
	// A5 = Enable

	GPIOA->MODER |= GPIO5_MODE_OUTPUT | GPIO4_MODE_ANALOG;
}

// Set next update to happen in x seconds
void battery_setUpdate(uint8_t secs)
{
	changeCount = secs;
	lastSecs = TIME_SECONDS(rtc_get());
}

void battery_update()
{
	// See if seconds has changed
	uint8_t secs = TIME_SECONDS(rtc_get());
	if(lastSecs == secs)
		return;
	lastSecs = secs;

	if(changeCount)
	{
		changeCount--;
		return;
	}

	battery_setUpdate(UPDATE_INTERVAL);
	battery_updateNow();
}

// Update voltage
void battery_updateNow()
{
	// Enable P-MOSFET
	GPIOA->ODR |= GPIO_ODR_5;

	// Wait a bit for things to turn on
	// Around 88us @ 8MHz
	for(uint8_t i=0;i<100;i++)
		__asm__ __volatile__ ("nop");

	// Get ADC value
	adc_enable();
	uint32_t adc = adc_read(ADC_CHANNEL);
	adc_disable();

	// Turn off MOSFET
	GPIOA->ODR &= ~GPIO_ODR_5;

	// Convert ADC value to voltage
	voltage = ((uint32_t)adc * MAX_VOLTAGE) / MAX_ADCVAL;

//	PRINTD("Batt: %lu\n", voltage);
}

// Get voltage
uint32_t battery_voltage()
{
	return voltage;
}
