/*
 * Project: SafeBee GPS Tracker
 * Author: Zak Kemble, contact@zakkemble.net
 * Copyright: (C) 2019 by Zak Kemble
 * License: 
 * Web: 
 */

#include "inc/common.h"

// Super simple EEPROM emulation using FLASH
// Only uses 1 page, no wear leveling and all that stuff
// ST app note AN4061

void eeprom_emu_init()
{

}

void eeprom_emu_update(void* dest, void* src, uint8_t len)
{
	while(FLASH->SR & FLASH_SR_BSY); // Wait for flash

	// Unlock flash
	if(FLASH->CR & FLASH_CR_LOCK)
	{
		FLASH->KEYR = FLASH_KEY1;
		FLASH->KEYR = FLASH_KEY2;
	}


	// Erase page
	// ~30ms
	FLASH->CR |= FLASH_CR_PER; // Allow page erase
	FLASH->AR = (uint32_t)dest; // Set page to erase
	FLASH->CR |= FLASH_CR_STRT; // Start erasing WARNING: This might halt the CPU (including interrupts)
	while (FLASH->SR & FLASH_SR_BSY); // Wait for finish
	if(FLASH->SR & FLASH_SR_EOP) // Check OK
		FLASH->SR = FLASH_SR_EOP;
	else
	{
		// Error
	}
	FLASH->CR &= ~FLASH_CR_PER; // Disallow page erase


	// Write data
	// ~53.5us per 16-bit
	FLASH->CR |= FLASH_CR_PG; // Allow programming
	for(uint8_t i=0;i<len;i++)
	{
		*(__IO uint16_t*)(dest + (i*2)) = ((uint16_t*)src)[i]; // Set data
		while(FLASH->SR & FLASH_SR_BSY); // Wait for finish
		if(FLASH->SR & FLASH_SR_EOP) // Check OK
			FLASH->SR = FLASH_SR_EOP;
		else
		{
			// Error
		}
	}
	FLASH->CR &= ~FLASH_CR_PG; // Disallow programming


	FLASH->CR |= FLASH_CR_LOCK; // Lock flash
}
