/*
 * Project: SafeBee GPS Tracker
 * Author: Zak Kemble, contact@zakkemble.net
 * Copyright: (C) 2019 by Zak Kemble
 * License: 
 * Web: 
 */

#include "inc/common.h"

// GPIOB
#define ACCEL_SCL 6
#define ACCEL_SDA 7

// GPIOA
#define GPS_SCL 9
#define GPS_SDA 10

static void init(void)
{
	RCC->APB1ENR |= RCC_APB1ENR_I2C1EN; // Enable clock
	RCC->APB1RSTR |= RCC_APB1RSTR_I2C1RST; // Pulse reset
	RCC->APB1RSTR &= ~RCC_APB1RSTR_I2C1RST; // Pulse reset

	I2C1->TIMINGR = (2<<I2C_TIMINGR_SCLDEL_Pos) | (0<<I2C_TIMINGR_SDADEL_Pos) | (29<<I2C_TIMINGR_SCLH_Pos) | (43<<I2C_TIMINGR_SCLL_Pos); // ~100KHz
	
//	I2C1->TIMEOUTR = 10<<I2C_TIMEOUTR_TIMEOUTA_Pos;

	RCC->APB1ENR &= ~RCC_APB1ENR_I2C1EN;
}
/*
static void clearBus(void)
{
	// Clear accel I2C bus
	// TODO GPS

	GPIOB->MODER |= GPIO_MODER_MODER6_0;
	GPIOB->OTYPER |= GPIO_OTYPER_OT_6;
	for(uint8_t i=0;i<9;i++)
	{
		GPIOB->BSRR = GPIO_ODR_6;
		delay(1);
		GPIOB->BRR = GPIO_ODR_6;
		delay(1);
	}
	GPIOB->MODER &= ~GPIO_MODER_MODER6_0;
}
*/
void i2c_init()
{
	//clearBus();

	// Accel
	//GPIOB->MODER |= GPIO_MODE(ACCEL_SCL, ALT) | GPIO_MODE(ACCEL_SDA, ALT); // Mode: AF
	GPIOB->PUPDR |= GPIO_PUPD(ACCEL_SCL, UP) | GPIO_PUPD(ACCEL_SDA, UP); // Pullup
	GPIOB->AFR[0] |= GPIO_AFL(ACCEL_SCL, 1) | GPIO_AFL(ACCEL_SDA, 1); // AF 1
	GPIOB->OTYPER |= GPIO_OTYPE(ACCEL_SCL, OD) | GPIO_OTYPE(ACCEL_SDA, OD); // Open drain
	GPIOB->ODR |= GPIO_OUT(ACCEL_SCL, HIGH) | GPIO_OUT(ACCEL_SDA, HIGH); // HIGH

	// GPS
	GPIOA->AFR[1] |= GPIO_AFH(GPS_SCL, 4) | GPIO_AFH(GPS_SDA, 4); // AF 4
	GPIOA->OTYPER |= GPIO_OTYPE(GPS_SCL, OD) | GPIO_OTYPE(GPS_SDA, OD); // Open drain

	i2c_gps(I2C_GPS_OFF);

	init();
}

void i2c_gps(i2c_gps_t state)
{
	// 0 = off (output low, no pullups)
	// 1 = on (OD output, pullups)
	// 2 = on + data (AF, OD output, pullups)
	
	switch(state)
	{
		case I2C_GPS_OFF:
			// GPS
			GPIOA->ODR &= (GPIO_OUT(GPS_SCL, MASK) & GPIO_OUT(GPS_SDA, MASK)); // LOW
			GPIOA->MODER = (GPIOA->MODER & (GPIO_MODE(GPS_SCL, MASK) & GPIO_MODE(GPS_SDA, MASK))) | GPIO_MODE(GPS_SCL, OUTPUT) | GPIO_MODE(GPS_SDA, OUTPUT); // Mode: OUTPUT
			GPIOA->PUPDR = (GPIOA->PUPDR & (GPIO_PUPD(GPS_SCL, MASK) & GPIO_PUPD(GPS_SDA, MASK))); // Pullup disable
			// Will now be open-drain OUTPUT LOW, no pullup

			// Accel
			GPIOB->MODER = (GPIOB->MODER & (GPIO_MODE(ACCEL_SCL, MASK) & GPIO_MODE(ACCEL_SDA, MASK))) | GPIO_MODE(ACCEL_SCL, ALT) | GPIO_MODE(ACCEL_SDA, ALT);
			break;
		case I2C_GPS_ON:
			// GPS
			GPIOA->ODR |= GPIO_OUT(GPS_SCL, HIGH) | GPIO_OUT(GPS_SDA, HIGH); // HIGH
			GPIOA->PUPDR = (GPIOA->PUPDR & (GPIO_PUPD(GPS_SCL, MASK) & GPIO_PUPD(GPS_SDA, MASK))) | GPIO_PUPD(GPS_SCL, UP) | GPIO_PUPD(GPS_SDA, UP); // Pullup enable
			GPIOA->MODER = (GPIOA->MODER & (GPIO_MODE(GPS_SCL, MASK) & GPIO_MODE(GPS_SDA, MASK))) | GPIO_MODE(GPS_SCL, OUTPUT) | GPIO_MODE(GPS_SDA, OUTPUT); // Mode: OUTPUT
			// Will now be open-drain OUTPUT HIGH, pullup

			// Accel
			GPIOB->MODER = (GPIOB->MODER & (GPIO_MODE(ACCEL_SCL, MASK) & GPIO_MODE(ACCEL_SDA, MASK))) | GPIO_MODE(ACCEL_SCL, ALT) | GPIO_MODE(ACCEL_SDA, ALT);
			break;
		case I2C_GPS_DATA:
			// GPS
			GPIOA->PUPDR = (GPIOA->PUPDR & (GPIO_PUPD(GPS_SCL, MASK) & GPIO_PUPD(GPS_SDA, MASK))) | GPIO_PUPD(GPS_SCL, UP) | GPIO_PUPD(GPS_SDA, UP); // Pullup enable
			GPIOA->MODER = (GPIOA->MODER & (GPIO_MODE(GPS_SCL, MASK) & GPIO_MODE(GPS_SDA, MASK))) | GPIO_MODE(GPS_SCL, ALT) | GPIO_MODE(GPS_SDA, ALT); // Mode: ALT
			// Will now be open-drain I2C, pullup

			// Accel
			GPIOB->MODER = (GPIOB->MODER & (GPIO_MODE(ACCEL_SCL, MASK) & GPIO_MODE(ACCEL_SDA, MASK))) | GPIO_MODE(ACCEL_SCL, OUTPUT) | GPIO_MODE(ACCEL_SDA, OUTPUT); // Mode: OUTPUT
			break;
		default:
			break;
	}
}

static void start(uint8_t address, uint8_t len)
{
//	I2C1->TIMEOUTR &= ~I2C_TIMEOUTR_TIMOUTEN;
//	I2C1->TIMEOUTR &= ~I2C_TIMEOUTR_TIDLE;
//	I2C1->TIMEOUTR |= I2C_TIMEOUTR_TIMOUTEN;

	I2C1->ICR |= I2C_ICR_TIMOUTCF|I2C_ICR_ARLOCF|I2C_ICR_BERRCF|I2C_ICR_STOPCF;
	
	I2C1->CR1 |= I2C_CR1_PE;

	uint32_t reg = I2C1->CR2;
	reg &= ~(I2C_CR2_NBYTES | I2C_CR2_RD_WRN | I2C_CR2_SADD); // Clear bits
	reg |= (len<<I2C_CR2_NBYTES_Pos) | I2C_CR2_START | ((address & 0x01) ? I2C_CR2_RD_WRN : 0) | (address<<I2C_CR2_SADD_Pos);
	I2C1->CR2 = reg;
}

static void stop(void)
{
//	I2C1->TIMEOUTR &= ~I2C_TIMEOUTR_TIMOUTEN;
//	I2C1->TIMEOUTR |= I2C_TIMEOUTR_TIDLE|I2C_TIMEOUTR_TIMOUTEN;

	I2C1->CR2 |= I2C_CR2_STOP;
	while(!(I2C1->ISR & (I2C_ISR_STOPF|I2C_ISR_TIMEOUT|I2C_ISR_ARLO|I2C_ISR_BERR)));

	I2C1->CR1 &= ~I2C_CR1_PE;
}

uint8_t i2c_write(uint8_t address, void* data, uint8_t len, uint8_t doStop)
{
	RCC->APB1ENR |= RCC_APB1ENR_I2C1EN;

	start(address, len);

	uint8_t i = 0;
	while(1)
	{
		uint32_t reg = I2C1->ISR;
		if(reg & (I2C_ISR_NACKF))
			break;
		else if(reg & (I2C_ISR_TIMEOUT|I2C_ISR_ARLO|I2C_ISR_BERR)) // Bus error
		{
			I2C1->CR1 &= ~I2C_CR1_PE;
			doStop = 0;
			PRINTD("I2C Bus error! %lu\n", reg);
			//clearBus(); // TODO
			//return i;
			break;
		}
		else if(i < len && (reg & I2C_ISR_TXIS))
			I2C1->TXDR = ((uint8_t*)data)[i++];
		else if(i >= len && (reg & I2C_ISR_TC)) // Transfer complete
			break;
	}

	if(doStop)
		stop();

	RCC->APB1ENR &= ~RCC_APB1ENR_I2C1EN;

	return i;
}

uint8_t i2c_read(uint8_t address, void* data, uint8_t len)
{
	RCC->APB1ENR |= RCC_APB1ENR_I2C1EN;

	start(address | 0x01, len);

	uint8_t doStop = 1;
	uint8_t i = 0;
	while(1)
	{
		uint32_t reg = I2C1->ISR;
		if(reg & (I2C_ISR_NACKF))
			break;
		else if(reg & (I2C_ISR_TIMEOUT|I2C_ISR_ARLO|I2C_ISR_BERR)) // Bus error
		{
			I2C1->CR1 &= ~I2C_CR1_PE;
			doStop = 0;
			PRINTD("I2C Bus error! %lu\n", reg);
			//clearBus(); // TODO
			//return i;
			break;
		}
		else if(i < len && (reg & I2C_ISR_RXNE))
			((uint8_t*)data)[i++] = I2C1->RXDR;
		else if(i >= len && (reg & I2C_ISR_TC)) // Transfer complete
			break;
	}

	if(doStop)
		stop();

	RCC->APB1ENR &= ~RCC_APB1ENR_I2C1EN;
	
	return i;
}
