/*
 * Project: SafeBee GPS Tracker
 * Author: Zak Kemble, contact@zakkemble.net
 * Copyright: (C) 2019 by Zak Kemble
 * License: 
 * Web: 
 */

#include "inc/common.h"

#define LED_COUNT	2

typedef struct {
	millis_t onPeriod;
	millis_t offPeriod;
	millis_t timer;
	//uint8_t count;
	//uint8_t counter;
	uint8_t state;
	uint8_t run;
} led_t;

static led_t leds[LED_COUNT];

void led_init()
{

}

void led_update()
{
	millis_t now = millis();
	for(uint8_t i=0;i<LED_COUNT;i++)
	{
		led_t* led = &leds[i];

		if(!led->run)
			continue;

		millis_t period = (led->state == LED_ON) ? led->onPeriod : led->offPeriod;
		if(now - led->timer >= period)
		{
			led->timer = now;
			if(led->state == LED_ON)
			{
				led->state = LED_OFF;
				setLED(i, LED_OFF);

				//led->counter++;
				//if(led->count && led->counter >= led->count)
				//	led->run = 0;
			}
			else
			{
				led->state = LED_ON;
				setLED(i, LED_ON);
			}
		}
	}
}

void led_on(uint8_t ledIdx)
{
	memset(&leds[ledIdx], 0, sizeof(led_t));
	setLED(ledIdx, LED_ON);
}

void led_off(uint8_t ledIdx)
{
	memset(&leds[ledIdx], 0, sizeof(led_t));
	setLED(ledIdx, LED_OFF);
}

void led_set(uint8_t ledIdx, millis_t onPeriod, millis_t offPeriod)
{
	led_t* led = &leds[ledIdx];

	if(onPeriod != led->onPeriod || offPeriod != led->offPeriod) // Don't want to keep resetting the timer
	{
		led->onPeriod = onPeriod;
		led->offPeriod = offPeriod;
		//led->count = 0;
		//led->counter = 0;
		led->timer = millis();

		// If onPeriod or offPeriod are 0 then we only want a single state, no flashing
		led->state = onPeriod ? LED_ON : LED_OFF;
		led->run = (onPeriod && offPeriod);
		setLED(ledIdx, led->state);
	}
}

void led_sleep()
{
	setLED(0, LED_OFF);
	setLED(1, LED_OFF);
}
