/*
 * Project: SafeBee GPS Tracker
 * Author: Zak Kemble, contact@zakkemble.net
 * Copyright: (C) 2019 by Zak Kemble
 * License: 
 * Web: 
 */

#include "inc/common.h"

void setLED(uint8_t led, uint8_t state)
{
	if(state == LED_OFF)
		GPIOF->ODR &= ~(1<<led);
	else if(state == LED_ON)
		GPIOF->ODR |= (1<<led);
	else if(state == LED_TOGGLE)
		GPIOF->ODR ^= (1<<led);
}

uint8_t prefix(const char* pre, const char* str)
{
    return strncmp(pre, str, strlen(pre)) == 0;
}

#define HEXLEN 2
uint8_t hex2dec(char* str)
{
	// Smaller version of strtoul(str, NULL, 16)
	// Nabbed from https://stackoverflow.com/a/130552
	uint8_t val = 0;
	for(uint8_t i=0;i<HEXLEN;i++)
		val += (str[i] - ((str[i] <= 57) ? 48 : 55)) * (1<<(4 * (HEXLEN - 1 - i)));
	return val;
}

// Similar to strsep()
// We use this function instead of strtok() as strtok will skip the field if there are no characters between the delimiters.
// Delimitors inside quotes will be ignored.
// It also does not store any kind of internal state.
// Example string: "blah1,,blah3,blah4,"
// strtok() will return pointers to blah1, blah3 and blah4, but nothing for the 2nd and 5th empty fields
char* getNextField(char** str)
{
	if(*str == NULL)
		return NULL;

	uint8_t insideQuotes = 0;

	char* token = *str;
	for(uint8_t i=0;;i++)
	{
		char c = (*str)[i];
		if(c == '\0')
		{
			(*str) = NULL;
			break;
		}
		else if(c == '"')
		{
			insideQuotes = !insideQuotes;
		}
		else if(!insideQuotes && (c == ',' || c == '*'))
		{
			(*str)[i] = '\0';
			(*str) = &((*str)[i + 1]);
			break;
		}
	}

	return token;
}
/*
void remove_chars(char* str, char* c)
{
	char* cPtr = c;
	char* pr = str;
	char* pw = str;
	while (*pr)
	{
		*pw = *pr++;

		c = cPtr;
		while(*c)
		{
			if(*pw == *c)
				break;
			c++;
		}

		pw += (*c == '\0');
	}
	*pw = '\0';
}
*/
